Module: CancerRegistryReportingTestKit::SearchTest
- Extended by:
- Forwardable
- Includes:
- FHIRResourceNavigation
- Included in:
- MedicationAdministrationSearchTest, MedicationRequestSearchTest, PrimaryCancerConditionSearchTest, RadiotherapyProcedureSearchTest, SecondaryCancerConditionSearchTest, TNMDistantMetastasesCategorySearchTest, TNMPrimaryTumorCategorySearchTest, TNMRegionalNodesCategorySearchTest, TNMStageGroupSearchTest
- Defined in:
- lib/cancer_registry_reporting_test_kit/search_test.rb
Constant Summary
Constants included from FHIRResourceNavigation
FHIRResourceNavigation::DAR_EXTENSION_URL, FHIRResourceNavigation::PRIMITIVE_DATA_TYPES
Instance Attribute Summary collapse
-
#manual_search_params ⇒ Object
Returns the value of attribute manual_search_params.
Instance Method Summary collapse
- #all_comparator_searches_tested? ⇒ Boolean
- #all_provenance_revinclude_search_params ⇒ Object
- #all_scratch_resources ⇒ Object
- #all_search_params ⇒ Object
- #all_search_params_present?(params) ⇒ Boolean
- #all_search_variants_tested? ⇒ Boolean
- #any_valid_search_params?(search_params) ⇒ Boolean
- #array_of_codes(array) ⇒ Object
- #cant_resolve_next_bundle_message(link) ⇒ Object
-
#check_resource_against_params(resource, params) ⇒ Object
RESULT CHECKING ####.
- #check_search_response ⇒ Object
- #date_comparator_value(comparator, date) ⇒ Object
- #default_search_values(param_name) ⇒ Object
- #element_has_valid_value?(element, include_system) ⇒ Boolean
- #empty_search_params_message(empty_search_params) ⇒ Object
- #fetch_all_bundled_resources(reply_handler: nil, max_pages: 20, additional_resource_types: [], resource_type: self.resource_type, params: nil) ⇒ Object
- #filter_conditions(resources) ⇒ Object
- #filter_devices(resources) ⇒ Object
- #fixed_value_search_param_name ⇒ Object
- #fixed_value_search_param_values ⇒ Object
- #fixed_value_search_params(value, patient_id) ⇒ Object
- #initial_search_variant_test_records ⇒ Object
- #is_reference_match?(reference, local_reference) ⇒ Boolean
- #manual_search_param_hash(patient_id) ⇒ Object
- #manual_search_param_name ⇒ Object
- #manual_search_param_values ⇒ Object
- #manual_search_params_full(value, patient_id) ⇒ Object
- #no_resources_skip_message(resource_type = self.resource_type) ⇒ Object
- #patient_id_list ⇒ Object
- #patient_id_param?(name) ⇒ Boolean
- #patient_search? ⇒ Boolean
- #perform_comparator_searches(params, patient_id) ⇒ Object
- #perform_multiple_or_search_test ⇒ Object
- #perform_post_search(get_search_resources, params) ⇒ Object
- #perform_reference_with_type_search(params, resource_count) ⇒ Object
- #perform_search(params, patient_id) ⇒ Object
- #perform_search_with_status(original_params, _patient_id, status_search_values: self.status_search_values, resource_type: self.resource_type) ⇒ Object
- #perform_search_with_system(params, patient_id) ⇒ Object
- #references_to_save(resource_type = nil) ⇒ Object
- #required_comparators(name) ⇒ Object
- #resource_matches_param?(resource, search_param_name, escaped_search_value, values_found = []) ⇒ Boolean
- #run_provenance_revinclude_search_test ⇒ Object
- #run_search_test ⇒ Object
- #save_delayed_references(resources, containing_resource_type = resource_type) ⇒ Object
- #save_resource_reference(resource_type, reference) ⇒ Object
- #scratch_resources_for_patient(patient_id) ⇒ Object
- #search_and_check_response(params, resource_type = self.resource_type) ⇒ Object
- #search_param_paths(name) ⇒ Object
- #search_param_value(name, resource, include_system: false) ⇒ Object
- #search_params_tag(params) ⇒ Object
- #search_params_with_values(search_param_names, patient_id, include_system: false) ⇒ Object
- #search_variant_test_records ⇒ Object
- #status_search_param_name ⇒ Object
- #status_search_values ⇒ Object
- #tags(params) ⇒ Object
- #test_medication_inclusion(base_resources, params, patient_id) ⇒ Object
- #unable_to_resolve_params_message ⇒ Object
- #unescape_search_value(value) ⇒ Object
Methods included from FHIRResourceNavigation
#find_a_value_at, #find_slice_via_discriminator, #get_next_value, #get_primitive_type_value, #resolve_path, #verify_slice_by_values
Instance Attribute Details
#manual_search_params ⇒ Object
Returns the value of attribute manual_search_params.
27 28 29 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 27 def manual_search_params @manual_search_params end |
Instance Method Details
#all_comparator_searches_tested? ⇒ Boolean
227 228 229 230 231 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 227 def all_comparator_searches_tested? return true if params_with_comparators.blank? Set.new(params_with_comparators) == search_variant_test_records[:comparator_searches] end |
#all_provenance_revinclude_search_params ⇒ Object
49 50 51 52 53 54 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 49 def all_provenance_revinclude_search_params @all_provenance_revinclude_search_params ||= all_search_params.transform_values! do |params_list| params_list.map { |params| params.merge(_revinclude: 'Provenance:target') } end end |
#all_scratch_resources ⇒ Object
465 466 467 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 465 def all_scratch_resources scratch_resources[:all] ||= [] end |
#all_search_params ⇒ Object
29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 29 def all_search_params @all_search_params ||= patient_id_list.each_with_object({}) do |patient_id, params| params[patient_id] ||= [] new_params = if manual_search_params manual_search_param_hash(patient_id) elsif fixed_value_search? fixed_value_search_param_values.map { |value| fixed_value_search_params(value, patient_id) } else [search_params_with_values(search_param_names, patient_id)] end new_params.reject! do |params| params.any? { |_key, value| value.blank? } end params[patient_id].concat(new_params) end end |
#all_search_params_present?(params) ⇒ Boolean
572 573 574 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 572 def all_search_params_present?(params) params.all? { |_name, value| value.present? } end |
#all_search_variants_tested? ⇒ Boolean
222 223 224 225 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 222 def all_search_variants_tested? search_variant_test_records.all? { |_variant, tested| tested.present? } && all_comparator_searches_tested? end |
#any_valid_search_params?(search_params) ⇒ Boolean
56 57 58 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 56 def any_valid_search_params?(search_params) search_params.any? { |_patient_id, params| params.present? } end |
#array_of_codes(array) ⇒ Object
576 577 578 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 576 def array_of_codes(array) array.map { |name| "`#{name}`" }.join(', ') end |
#cant_resolve_next_bundle_message(link) ⇒ Object
646 647 648 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 646 def (link) "Could not resolve next bundle: #{link}" end |
#check_resource_against_params(resource, params) ⇒ Object
RESULT CHECKING ####
765 766 767 768 769 770 771 772 773 774 775 776 777 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 765 def check_resource_against_params(resource, params) params.each do |name, escaped_search_value| values_found = [] search_value = unescape_search_value(escaped_search_value) match_found = resource_matches_param?(resource, name, escaped_search_value, values_found) assert match_found, "#{resource_type}/#{resource.id} did not match the search parameters:\n" \ "* Expected: #{unescape_search_value(search_value)}\n" \ "* Found: #{values_found.map(&:inspect).join(', ')}" end end |
#check_search_response ⇒ Object
202 203 204 205 206 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 202 def check_search_response assert_response_status(200) assert_resource_type(:bundle) # NOTE: how do we want to handle validating Bundles? end |
#date_comparator_value(comparator, date) ⇒ Object
233 234 235 236 237 238 239 240 241 242 243 244 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 233 def date_comparator_value(comparator, date) date = date.start || date.end if date.is_a? FHIR::Period case comparator when 'lt', 'le' comparator + (DateTime.xmlschema(date) + 1).xmlschema when 'gt', 'ge' comparator + (DateTime.xmlschema(date) - 1).xmlschema else # '' raise "Unsupported comparator '#{comparator}'" end end |
#default_search_values(param_name) ⇒ Object
348 349 350 351 352 353 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 348 def default_search_values(param_name) definition = .search_definitions[param_name] return [] if definition.blank? definition[:multiple_or] == 'SHALL' ? [definition[:values].join(',')] : Array.wrap(definition[:values]) end |
#element_has_valid_value?(element, include_system) ⇒ Boolean
711 712 713 714 715 716 717 718 719 720 721 722 723 724 725 726 727 728 729 730 731 732 733 734 735 736 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 711 def element_has_valid_value?(element, include_system) case element when FHIR::Reference element.reference.present? when FHIR::CodeableConcept if include_system coding = find_a_value_at(element, 'coding') { |coding| coding.code.present? && coding.system.present? } coding.present? else find_a_value_at(element, 'coding.code').present? end when FHIR::Identifier include_system ? element.value.present? && element.system.present? : element.value.present? when FHIR::Coding include_system ? element.code.present? && element.system.present? : element.code.present? when FHIR::HumanName (element.family || element.given&.first || element.text).present? when FHIR::Address (element.text || element.city || element.state || element.postalCode || element.country).present? when CancerRegistryReportingTestKit::PrimitiveType element.value.present? else true end end |
#empty_search_params_message(empty_search_params) ⇒ Object
584 585 586 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 584 def (empty_search_params) "Could not find values for the search parameters #{array_of_codes(empty_search_params.keys)}" end |
#fetch_all_bundled_resources(reply_handler: nil, max_pages: 20, additional_resource_types: [], resource_type: self.resource_type, params: nil) ⇒ Object
598 599 600 601 602 603 604 605 606 607 608 609 610 611 612 613 614 615 616 617 618 619 620 621 622 623 624 625 626 627 628 629 630 631 632 633 634 635 636 637 638 639 640 641 642 643 644 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 598 def fetch_all_bundled_resources( reply_handler: nil, max_pages: 20, additional_resource_types: [], resource_type: self.resource_type, params: nil ) page_count = 1 resources = [] bundle = resource until bundle.nil? || page_count == max_pages resources += bundle&.entry&.map { |entry| entry&.resource } next_bundle_link = bundle&.link&.find { |link| link.relation == 'next' }&.url reply_handler&.call(response) break if next_bundle_link.blank? reply = fhir_client.raw_read_url(next_bundle_link) store_request('outgoing', tags: (params)) { reply } = (next_bundle_link) assert_response_status(200) assert_valid_json(reply.body, ) bundle = fhir_client.parse_reply(FHIR::Bundle, fhir_client.default_format, reply) page_count += 1 end valid_resource_types = [resource_type, 'OperationOutcome'].concat(additional_resource_types) valid_resource_types << 'Medication' if %w[MedicationRequest MedicationDispense].include?(resource_type) invalid_resource_types = resources.reject { |entry| valid_resource_types.include? entry.resourceType } .map(&:resourceType) .uniq if invalid_resource_types.any? info "Received resource type(s) #{invalid_resource_types.join(', ')} in search bundle, " \ "but only expected resource types #{valid_resource_types.join(', ')}. " \ 'This is unusual but allowed if the server believes additional resource types are relevant.' end resources end |
#filter_conditions(resources) ⇒ Object
176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 176 def filter_conditions(resources) # HL7 JIRA FHIR-37917. US Core v5.0.1 does not required patient+category. # In order to distinguish which resources matches the current profile, Inferno has to manually filter # the result of first search, which is searching by patient. selected_categories = [] all_search_params.each do |key, array_of_param_hashes| categories = array_of_param_hashes.map { |param_hash| param_hash['category']} selected_categories.concat categories end selected_categories.uniq! resources.select! do |resource| resource.category.any? do |category| category.coding.any? do |coding| selected_categories.include? coding.code end end end end |
#filter_devices(resources) ⇒ Object
167 168 169 170 171 172 173 174 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 167 def filter_devices(resources) codes_to_include = implantable_device_codes&.split(',')&.map(&:strip) return resources if codes_to_include.blank? resources.select! do |resource| resource&.type&.coding&.any? { |coding| codes_to_include.include?(coding.code) } end end |
#fixed_value_search_param_name ⇒ Object
480 481 482 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 480 def fixed_value_search_param_name (search_param_names - ['patient']).first end |
#fixed_value_search_param_values ⇒ Object
495 496 497 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 495 def fixed_value_search_param_values .search_definitions[fixed_value_search_param_name.to_sym][:values] end |
#fixed_value_search_params(value, patient_id) ⇒ Object
499 500 501 502 503 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 499 def fixed_value_search_params(value, patient_id) search_param_names.each_with_object({}) do |name, params| params[name] = patient_id_param?(name) ? patient_id : value end end |
#initial_search_variant_test_records ⇒ Object
212 213 214 215 216 217 218 219 220 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 212 def initial_search_variant_test_records {}.tap do |records| records[:post_variant] = false if test_post_search? records[:medication_inclusion] = false if test_medication_inclusion? records[:reference_variants] = false if test_reference_variants? records[:token_variants] = false if token_search_params.present? records[:comparator_searches] = Set.new if params_with_comparators.present? end end |
#is_reference_match?(reference, local_reference) ⇒ Boolean
460 461 462 463 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 460 def is_reference_match?(reference, local_reference) regex_pattern = %r{^(#{Regexp.escape(local_reference)}|\S+/#{Regexp.escape(local_reference)}(?:[/|]\S+)*)$} reference.match?(regex_pattern) end |
#manual_search_param_hash(patient_id) ⇒ Object
484 485 486 487 488 489 490 491 492 493 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 484 def manual_search_param_hash(patient_id) single_patient_manual_search_params = [patient_id] + manual_search_params split_values = single_patient_manual_search_params.map { |value| value.split(',') } combinations = split_values[0].product(*split_values[1..]) combinations.map do |combination| search_param_names.zip(combination).to_h end end |
#manual_search_param_name ⇒ Object
505 506 507 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 505 def manual_search_param_name (search_param_names - ['patient']).first end |
#manual_search_param_values ⇒ Object
509 510 511 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 509 def manual_search_param_values .search_definitions[manual_search_param_name.to_sym][:values] end |
#manual_search_params_full(value, patient_id) ⇒ Object
513 514 515 516 517 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 513 def manual_search_params_full(value, patient_id) search_param_names.each_with_object({}) do |name, params| params[name] = patient_id_param?(name) ? patient_id : value end end |
#no_resources_skip_message(resource_type = self.resource_type) ⇒ Object
588 589 590 591 592 593 594 595 596 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 588 def (resource_type = self.resource_type) msg = "No #{resource_type} resources appear to be available" if resource_type == 'Device' && implantable_device_codes.present? msg.concat(" with the following Device Type Code filter: #{implantable_device_codes}") end "#{msg}. Please use patients with more information" end |
#patient_id_list ⇒ Object
547 548 549 550 551 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 547 def patient_id_list return [nil] unless respond_to? :patient_ids patient_ids.split(',').map(&:strip) end |
#patient_id_param?(name) ⇒ Boolean
557 558 559 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 557 def patient_id_param?(name) name == 'patient' || (name == '_id' && resource_type == 'Patient') end |
#patient_search? ⇒ Boolean
553 554 555 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 553 def patient_search? search_param_names.any? { |name| patient_id_param? name } end |
#perform_comparator_searches(params, patient_id) ⇒ Object
255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 255 def perform_comparator_searches(params, patient_id) params_with_comparators.each do |name| next if search_variant_test_records[:comparator_searches].include? name required_comparators(name).each do |comparator| paths = search_param_paths(name).first date_element = find_a_value_at(scratch_resources_for_patient(patient_id), paths) params_with_comparator = params.merge(name => date_comparator_value(comparator, date_element)) search_and_check_response(params_with_comparator) fetch_all_bundled_resources(params: params_with_comparator).each do |resource| check_resource_against_params(resource, params_with_comparator) if resource.resourceType == resource_type end end search_variant_test_records[:comparator_searches] << name end end |
#perform_multiple_or_search_test ⇒ Object
355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 397 398 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 355 def perform_multiple_or_search_test resolved_one = false all_search_params.each do |patient_id, params_list| next unless params_list.present? search_params = params_list.first existing_values = {} missing_values = {} multiple_or_search_params.each do |param_name| search_value = default_search_values(param_name.to_sym) search_params = search_params.merge(param_name.to_s => search_value) existing_values[param_name.to_sym] = scratch_resources_for_patient(patient_id).map(¶m_name.to_sym).compact.uniq end # skip patient without multiple-or values next if existing_values.values.any?(&:empty?) resolved_one = true search_and_check_response(search_params) resources_returned = fetch_all_bundled_resources(params: search_params) .select { |resource| resource.resourceType == resource_type } multiple_or_search_params.each do |param_name| missing_values[param_name.to_sym] = existing_values[param_name.to_sym] - resources_returned.map(¶m_name.to_sym) end = missing_values .reject { |_param_name, missing_value| missing_value.empty? } .map { |param_name, missing_value| "#{missing_value.join(',')} values from #{param_name}" } .join(' and ') assert .blank?, "Could not find #{} in any of the resources returned for Patient/#{patient_id}" break if resolved_one end end |
#perform_post_search(get_search_resources, params) ⇒ Object
146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 146 def perform_post_search(get_search_resources, params) fhir_search resource_type, params:, search_method: :post check_search_response post_search_resources = fetch_all_bundled_resources.select { |resource| resource.resourceType == resource_type } filter_conditions(post_search_resources) if resource_type == 'Condition' filter_devices(post_search_resources) if resource_type == 'Device' get_resource_count = get_search_resources.length post_resource_count = post_search_resources.length search_variant_test_records[:post_variant] = true assert get_resource_count == post_resource_count, 'Expected search by POST to return the same results as search by GET, ' \ "but GET search returned #{get_resource_count} resources, and POST search " \ "returned #{post_resource_count} resources." end |
#perform_reference_with_type_search(params, resource_count) ⇒ Object
275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 275 def perform_reference_with_type_search(params, resource_count) return if resource_count.zero? return if search_variant_test_records[:reference_variants] new_search_params = params.merge('patient' => "Patient/#{params['patient']}") search_and_check_response(new_search_params) reference_with_type_resources = fetch_all_bundled_resources(params: new_search_params) .select { |resource| resource.resourceType == resource_type } filter_conditions(reference_with_type_resources) if resource_type == 'Condition' && .version == 'v5.0.1' filter_devices(reference_with_type_resources) if resource_type == 'Device' new_resource_count = reference_with_type_resources.count assert new_resource_count == resource_count, "Expected search by `#{params['patient']}` to to return the same results as searching " \ "by `#{new_search_params['patient']}`, but found #{resource_count} resources with " \ "`#{params['patient']}` and #{new_resource_count} with `#{new_search_params['patient']}`" search_variant_test_records[:reference_variants] = true end |
#perform_search(params, patient_id) ⇒ Object
102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 102 def perform_search(params, patient_id) params.each do |key, value| if key == "date" || key == "effective-time" params[key] = ["ge#{value}", "le#{value}"] end end fhir_search resource_type, params:, tags: (params) perform_search_with_status(params, patient_id) if response[:status] == 400 && possible_status_search? check_search_response resources_returned = fetch_all_bundled_resources(params:).select { |resource| resource.resourceType == resource_type } return [] if resources_returned.blank? perform_comparator_searches(params, patient_id) if params_with_comparators.present? filter_conditions(resources_returned) if resource_type == 'Condition' filter_devices(resources_returned) if resource_type == 'Device' if first_search? all_scratch_resources.concat(resources_returned).uniq! scratch_resources_for_patient(patient_id).concat(resources_returned).uniq! end # resources_returned.each do |resource| # check_resource_against_params(resource, params) # end save_delayed_references(resources_returned) if saves_delayed_references? return resources_returned if all_search_variants_tested? perform_post_search(resources_returned, params) if test_post_search? test_medication_inclusion(resources_returned, params, patient_id) if test_medication_inclusion? perform_reference_with_type_search(params, resources_returned.count) if test_reference_variants? perform_search_with_system(params, patient_id) if token_search_params.present? resources_returned end |
#perform_search_with_status(original_params, _patient_id, status_search_values: self.status_search_values, resource_type: self.resource_type) ⇒ Object
317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 317 def perform_search_with_status( original_params, _patient_id, status_search_values: self.status_search_values, resource_type: self.resource_type ) assert resource.is_a?(FHIR::OperationOutcome), 'Server returned a status of 400 without an OperationOutcome' # TODO: warn about documenting status requirements status_search_values.flat_map do |status_value| search_params = original_params.merge("#{status_search_param_name}": status_value) search_and_check_response(search_params) entries = resource.entry.select { |entry| entry.resource.resourceType == resource_type } if entries.present? original_params.merge!("#{status_search_param_name}": status_value) break end end end |
#perform_search_with_system(params, patient_id) ⇒ Object
299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 299 def perform_search_with_system(params, patient_id) return if search_variant_test_records[:token_variants] new_search_params = search_params_with_values(token_search_params, patient_id, include_system: true) return if new_search_params.any? { |_name, value| value.blank? } search_params = params.merge(new_search_params) search_and_check_response(search_params) resources_returned = fetch_all_bundled_resources(params: search_params) .select { |resource| resource.resourceType == resource_type } assert resources_returned.present?, 'No resources were returned when searching by `system|code`' search_variant_test_records[:token_variants] = true end |
#references_to_save(resource_type = nil) ⇒ Object
475 476 477 478 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 475 def references_to_save(resource_type = nil) = resource_type == 'Provenance' ? : .delayed_references end |
#required_comparators(name) ⇒ Object
246 247 248 249 250 251 252 253 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 246 def required_comparators(name) .search_definitions .dig(name.to_sym, :comparators) .select { |_comparator, expectation| expectation == 'SHALL' } .keys .map(&:to_s) end |
#resource_matches_param?(resource, search_param_name, escaped_search_value, values_found = []) ⇒ Boolean
783 784 785 786 787 788 789 790 791 792 793 794 795 796 797 798 799 800 801 802 803 804 805 806 807 808 809 810 811 812 813 814 815 816 817 818 819 820 821 822 823 824 825 826 827 828 829 830 831 832 833 834 835 836 837 838 839 840 841 842 843 844 845 846 847 848 849 850 851 852 853 854 855 856 857 858 859 860 861 862 863 864 865 866 867 868 869 870 871 872 873 874 875 876 877 878 879 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 783 def resource_matches_param?(resource, search_param_name, escaped_search_value, values_found = []) search_value = unescape_search_value(escaped_search_value) paths = search_param_paths(search_param_name) match_found = false paths.each do |path| type = .search_definitions[search_param_name.to_sym][:type] resolve_path(resource, path).each do |value| values_found << if value.is_a? FHIR::Reference value.reference elsif value.is_a? CancerRegistryReportingTestKit::PrimitiveType value.value else value end end values_found.compact! match_found = case type when 'Period', 'date', 'instant', 'dateTime' values_found.any? { |date| validate_date_search(search_value, date) } when 'HumanName' # When a string search parameter refers to the types HumanName and Address, # the search covers the elements of type string, and does not cover elements such as use and period # https://www.hl7.org/fhir/search.html#string search_value_downcase = search_value.downcase values_found.any? do |name| name&.text&.downcase&.start_with?(search_value_downcase) || name&.family&.downcase&.start_with?(search_value_downcase) || name&.given&.any? { |given| given.downcase.start_with?(search_value_downcase) } || name&.prefix&.any? { |prefix| prefix.downcase.start_with?(search_value_downcase) } || name&.suffix&.any? { |suffix| suffix.downcase.start_with?(search_value_downcase) } end when 'Address' search_value_downcase = search_value.downcase values_found.any? do |address| address&.text&.downcase&.start_with?(search_value_downcase) || address&.city&.downcase&.start_with?(search_value_downcase) || address&.state&.downcase&.start_with?(search_value_downcase) || address&.postalCode&.downcase&.start_with?(search_value_downcase) || address&.country&.downcase&.start_with?(search_value_downcase) end when 'CodeableConcept' # FHIR token search (https://www.hl7.org/fhir/search.html#token): "When in doubt, servers SHOULD # treat tokens in a case-insensitive manner, on the grounds that including undesired data has # less safety implications than excluding desired behavior". codings = values_found.flat_map(&:coding) if search_value.include? '|' system = search_value.split('|').first code = search_value.split('|').last codings&.any? { |coding| coding.system == system && coding.code&.casecmp?(code) } else codings&.any? { |coding| coding.code&.casecmp?(search_value) } end when 'Coding' if search_value.include? '|' system = search_value.split('|').first code = search_value.split('|').last values_found.any? { |coding| coding.system == system && coding.code&.casecmp?(code) } else values_found.any? { |coding| coding.code&.casecmp?(search_value) } end when 'Identifier' if search_value.include? '|' values_found.any? { |identifier| "#{identifier.system}|#{identifier.value}" == search_value } else values_found.any? { |identifier| identifier.value == search_value } end when 'string' searched_values = search_value.downcase.split(/(?<!\\\\),/).map { |string| string.gsub('\\,', ',') } values_found.any? do |value_found| searched_values.any? { |searched_value| value_found.downcase.starts_with? searched_value } end else # searching by patient requires special case because we are searching by a resource identifier # references can also be URLs, so we may need to resolve those URLs if %w[subject patient].include? search_param_name.to_s id = search_value.split('Patient/').last possible_values = [id, "Patient/#{id}", "#{url}/Patient/#{id}"] values_found.any? do |reference| possible_values.include? reference end else search_values = search_value.split(/(?<!\\\\),/).map { |string| string.gsub('\\,', ',') } values_found.any? { |value_found| search_values.include? value_found } end end break if match_found end match_found end |
#run_provenance_revinclude_search_test ⇒ Object
60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 60 def run_provenance_revinclude_search_test # TODO: skip if not supported? skip_if !any_valid_search_params?(all_provenance_revinclude_search_params), provenance_resources = all_provenance_revinclude_search_params.flat_map do |_patient_id, params_list| params_list.flat_map do |params| fhir_search resource_type, params:, tags: (params) perform_search_with_status(params, patient_id) if response[:status] == 400 && possible_status_search? check_search_response # TODO: check that only provenance resources for resources matching # granular scopes returned fetch_all_bundled_resources(additional_resource_types: ['Provenance'], params:) .select { |resource| resource.resourceType == 'Provenance' } end end scratch_provenance_resources[:all] ||= [] scratch_provenance_resources[:all].concat(provenance_resources) save_delayed_references(provenance_resources, 'Provenance') skip_if provenance_resources.empty?, ('Provenance') end |
#run_search_test ⇒ Object
88 89 90 91 92 93 94 95 96 97 98 99 100 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 88 def run_search_test # TODO: skip if not supported? skip_if !any_valid_search_params?(all_search_params), resources_returned = all_search_params.flat_map do |patient_id, params_list| params_list.flat_map { |params| perform_search(params, patient_id) } end skip_if resources_returned.empty?, perform_multiple_or_search_test if multiple_or_search_params.present? end |
#save_delayed_references(resources, containing_resource_type = resource_type) ⇒ Object
744 745 746 747 748 749 750 751 752 753 754 755 756 757 758 759 760 761 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 744 def save_delayed_references(resources, containing_resource_type = resource_type) resources.each do |resource| references_to_save(containing_resource_type).each do |reference_to_save| resolve_path(resource, reference_to_save[:path]) .select do |reference| reference.is_a?(FHIR::Reference) && !reference.contained? && reference.reference.present? end .each do |reference| resource_type = reference.resource_class.name.demodulize need_to_save = reference_to_save[:resources].include?(resource_type) next unless need_to_save save_resource_reference(resource_type, reference) end end end end |
#save_resource_reference(resource_type, reference) ⇒ Object
738 739 740 741 742 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 738 def save_resource_reference(resource_type, reference) scratch[:references] ||= {} scratch[:references][resource_type] ||= Set.new scratch[:references][resource_type] << reference end |
#scratch_resources_for_patient(patient_id) ⇒ Object
469 470 471 472 473 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 469 def scratch_resources_for_patient(patient_id) return all_scratch_resources if patient_id.nil? scratch_resources[patient_id] ||= [] end |
#search_and_check_response(params, resource_type = self.resource_type) ⇒ Object
196 197 198 199 200 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 196 def search_and_check_response(params, resource_type = self.resource_type) fhir_search resource_type, params:, tags: (params) check_search_response end |
#search_param_paths(name) ⇒ Object
561 562 563 564 565 566 567 568 569 570 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 561 def search_param_paths(name) paths = .search_definitions[name.to_sym][:paths] if paths.first == 'class' paths[0] = 'local_class' elsif paths.first == 'method' paths[0] = 'local_method' end paths end |
#search_param_value(name, resource, include_system: false) ⇒ Object
650 651 652 653 654 655 656 657 658 659 660 661 662 663 664 665 666 667 668 669 670 671 672 673 674 675 676 677 678 679 680 681 682 683 684 685 686 687 688 689 690 691 692 693 694 695 696 697 698 699 700 701 702 703 704 705 706 707 708 709 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 650 def search_param_value(name, resource, include_system: false) paths = search_param_paths(name) search_value = nil paths.each do |path| element = find_a_value_at(resource, path) { |element| element_has_valid_value?(element, include_system) } search_value = case element when FHIR::Period if element.start.present? "gt#{(DateTime.xmlschema(element.start) - 1).xmlschema}" else end_datetime = get_fhir_datetime_range(element.end)[:end] "lt#{(end_datetime + 1).xmlschema}" end when FHIR::Reference element.reference when FHIR::CodeableConcept if include_system coding = find_a_value_at(element, 'coding') { |coding| coding.code.present? && coding.system.present? } "#{coding.system}|#{coding.code}" else find_a_value_at(element, 'coding.code') end when FHIR::Identifier include_system ? "#{element.system}|#{element.value}" : element.value when FHIR::Coding include_system ? "#{element.system}|#{element.code}" : element.code when FHIR::HumanName element.family || element.given&.first || element.text when FHIR::Address element.text || element.city || element.state || element.postalCode || element.country when CancerRegistryReportingTestKit::PrimitiveType element.value else if .version != 'v3.1.1' && .search_definitions[name.to_sym][:type] == 'date' && params_with_comparators&.include?(name) # convert date search to greath-than comparator search with correct precision # For all date search parameters: # Patient.birthDate does not mandate comparators so cannot be converted # Goal.target-date has day precision # All others have second + time offset precision if /^\d{4}(-\d{2})?$/.match?(element) || # YYYY or YYYY-MM (/^\d{4}-\d{2}-\d{2}$/.match?(element) && resource_type != 'Goal') # YYY-MM-DD AND Resource is NOT Goal "gt#{(DateTime.xmlschema(element) - 1).xmlschema}" else element end else element end end break if search_value.present? end search_value&.gsub(',', '\\,') end |
#search_params_tag(params) ⇒ Object
893 894 895 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 893 def search_params_tag(params) "#{resource_type}?#{params.keys.join('&')}" end |
#search_params_with_values(search_param_names, patient_id, include_system: false) ⇒ Object
519 520 521 522 523 524 525 526 527 528 529 530 531 532 533 534 535 536 537 538 539 540 541 542 543 544 545 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 519 def search_params_with_values(search_param_names, patient_id, include_system: false) resources = scratch_resources_for_patient(patient_id) if resources.empty? return search_param_names.each_with_object({}) do |name, params| value = patient_id_param?(name) ? patient_id : nil params[name] = value end end resources.each_with_object({}) do |resource, outer_params| results_from_one_resource = search_param_names.each_with_object({}) do |name, params| value = if patient_id_param?(name) patient_id else search_param_value(name, resource, include_system: include_system) end params[name] = value end outer_params.merge!(results_from_one_resource) # stop if all parameter values are found return outer_params if outer_params.all? { |_key, value| value.present? } end end |
#search_variant_test_records ⇒ Object
208 209 210 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 208 def search_variant_test_records @search_variant_test_records ||= initial_search_variant_test_records end |
#status_search_param_name ⇒ Object
339 340 341 342 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 339 def status_search_param_name @status_search_param_name ||= .search_definitions.keys.find { |key| key.to_s.include? 'status' } end |
#status_search_values ⇒ Object
344 345 346 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 344 def status_search_values default_search_values(status_search_param_name) end |
#tags(params) ⇒ Object
881 882 883 884 885 886 887 888 889 890 891 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 881 def (params) return nil unless config.[:tag_requests] return nil if params.blank? if %w[Condition DiagnosticReport DocumentReference Observation ServiceRequest].include? resource_type return [search_params_tag(params)] end nil end |
#test_medication_inclusion(base_resources, params, patient_id) ⇒ Object
400 401 402 403 404 405 406 407 408 409 410 411 412 413 414 415 416 417 418 419 420 421 422 423 424 425 426 427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 442 443 444 445 446 447 448 449 450 451 452 453 454 455 456 457 458 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 400 def test_medication_inclusion(base_resources, params, patient_id) return if search_variant_test_records[:medication_inclusion] scratch[:medication_resources] ||= {} scratch[:medication_resources][:all] ||= [] scratch[:medication_resources][patient_id] ||= [] scratch[:medication_resources][:contained] ||= [] base_resources_with_external_reference = base_resources .select { |request| request&.medicationReference&.present? } .reject { |request| request&.medicationReference&.reference&.start_with? '#' } contained_medications = base_resources .select { |request| request&.medicationReference&.reference&.start_with? '#' } .flat_map(&:contained) .select { |resource| resource.resourceType == 'Medication' } scratch[:medication_resources][:all] += contained_medications scratch[:medication_resources][patient_id] += contained_medications scratch[:medication_resources][:contained] += contained_medications return if base_resources_with_external_reference.blank? search_params = params.merge(_include: "#{resource_type}:medication") search_and_check_response(search_params) medications = fetch_all_bundled_resources(params: search_params) .select { |resource| resource.resourceType == 'Medication' } assert medications.present?, 'No Medications were included in the search results' included_medications = medications.map { |medication| "#{medication.resourceType}/#{medication.id}" } matched_base_resources = base_resources_with_external_reference.select do |base_resource| included_medications.any? do |medication_reference| is_reference_match?(base_resource.medicationReference.reference, medication_reference) end end not_matched_included_medications = included_medications.select do |medication_reference| matched_base_resources.none? do |base_resource| is_reference_match?(base_resource.medicationReference.reference, medication_reference) end end not_matched_included_medications_string = not_matched_included_medications.join(',') assert not_matched_included_medications.empty?, "No #{resource_type} references #{not_matched_included_medications_string} in the search result." medications.uniq!(&:id) scratch[:medication_resources][:all] += medications scratch[:medication_resources][patient_id] += medications search_variant_test_records[:medication_inclusion] = true end |
#unable_to_resolve_params_message ⇒ Object
580 581 582 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 580 def "Could not find values for all search params #{array_of_codes(search_param_names)}" end |
#unescape_search_value(value) ⇒ Object
779 780 781 |
# File 'lib/cancer_registry_reporting_test_kit/search_test.rb', line 779 def unescape_search_value(value) value&.gsub('\\,', ',') end |